#include <iostream>
#include <iomanip>
#include <limits>
#include <string>
using namespace std;

#include "Math.h"
using namespace arda::Vector;
using namespace arda::Matrix;

////////////////////////////////////////////////////////////////////////////////
void test_vector(void)
   {
   string s1, s2;

   // Vector::scale*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::scale*()" << endl;

   Vector2i v2i = { 1, 2 };
   cout << setw(40) << "v2i: " << getstring2v(v2i, s1) << endl;
   scale2v(v2i, 2);
   cout << setw(40) << "scale2v(v2i, 2): " << getstring2v(v2i, s1) << endl;
   scale2v(v2i, 2.4);
   cout << setw(40) << "scale2v(v2i, 2.4): " << getstring2v(v2i, s1) << endl;
   scale2v(scale2v(v2i, 2), 3);
   cout << setw(40) << "scale2v(scale2v(v2i, 2), 3): " << getstring2v(v2i, s1) << endl;


   Vector3f v3f = { 3.0, 4.1, 5.2 };
   cout << setw(40) << "v3f: " << getstring3v(v3f, s1) << endl;
   scale3v(v3f, 2.2f);
   cout << setw(40) << "scale3v(v3f,  2.2f): " << getstring3v(v3f, s1) << endl;
   scale3v(v3f, 2);
   cout << setw(40) << "scale3v(v2f, 2): " << getstring3v(v3f, s1) << endl;

   Vector4d v4d = { 6.0L, 7.1L, 8.2L, 9.3L };
   cout << setw(40) << "vd4: " << getstring4v(v4d, s1) << endl;
   scale4v(v4d, 2.0);
   cout << setw(40) << "scale4v(v4d, 2.0): " << getstring4v(v4d, s1) << endl;
   scale4v(v4d, 2);
   cout << setw(40) << "scale4v(v4d, 2): " << getstring4v(v4d, s1) << endl;
   }


   // Vector::dot*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::dot*()" << endl;

   Vector2i v2i[2] = { { 1, 2 }, { 3, 4 } };
   cout << setw(40) << "v2i[0]: " << getstring2v(v2i[0], s1) << endl;
   cout << setw(40) << "v2i[1]: " << getstring2v(v2i[1], s1) << endl;
   cout << setw(40) << "dot2v(v2i[0], v2i[1]): " << dot2v(v2i[0], v2i[1]) << endl;
   
   Vector3f v3f[3] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f } };
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   cout << setw(40) << "dot3v(v3f[0], v3f[1]): " << dot3v(v3f[0], v3f[1]) << endl;
   
   Vector4d v4d[3] = { { 1.1, 2.2, 3.3, 4.4 }, { 5.5, 6.6, 7.7, 8.8 } };
   cout << setw(40) << "v4d[0]: " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "v4d[1]: " << getstring4v(v4d[1], s1) <<  endl;
   cout << setw(40) << "dot4v(v4d[0], v4d[1]): " << dot4v(v4d[0], v4d[1]) << endl;
   
   }

   // Vector::length*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::length*()" << endl;

   Vector2i v2i = { 1, 2 };
   cout << setw(40) << "v2i: " << getstring2v(v2i, s1) << endl;
   cout << setw(40) << "length2v(v2i): " << length2v(v2i) << endl;
   
   Vector3f v3f = { 1.1f, 2.2f, 3.3f };
   cout << setw(40) << "v3f: " << getstring3v(v3f, s1) << endl;
   cout << setw(40) << "length3v(v3f): " << length3v(v3f) << endl;
   
   Vector4d v4d = { 1.1, 2.2, 3.3, 4.4 };
   cout << setw(40) << "v4d: " << getstring4v(v4d, s1) << endl;
   cout << setw(40) << "length4v(v4d): " << length4v(v4d) << endl;
   
   }

   // Vector::add*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::add*()" << endl;

   Vector2i v2i[3] = { { 1, 2 }, { 3, 4 } };
   cout << setw(40) << "v2i[0]: " << getstring2v(v2i[0], s1) << endl;
   cout << setw(40) << "v2i[1]: " << getstring2v(v2i[1], s1) << endl;
   add2v(v2i[0], v2i[1]);
   cout << setw(40) << "add2v(v2i[0], v2i[1]): " << getstring2v(v2i[0], s1) << endl;
   cout << setw(40) << "length2v(add2v(v2i[0], v2i[1])): " << length2v(add2v(v2i[0], v2i[1])) << endl;

   Vector3f v3f[3] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f } };
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   add3v(v3f[0], v3f[1]);
   cout << setw(40) << "add3v(v3f[0], v3f[1]): " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "length3v(add3v(v3f[0], v3f[1])): " << length3v(add3v(v3f[0], v3f[1])) << endl;

   Vector4d v4d[3] = { { 1.1, 2.2, 3.3, 4.4 }, { 5.5, 6.6, 7.7, 8.8 } };
   cout << setw(40) << "v4d[0]: " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "v4d[1]: " << getstring4v(v4d[1], s1) << endl;
   add4v(v4d[0], v4d[1]);
   cout << setw(40) << "add4v(v4d[0], v4d[1]): " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "length4v(add4v(v4d[0], v4d[1])): " << length4v(add4v(v4d[0], v4d[1])) << endl;

   }

   // Vector::normalize*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::normalize*()" << endl;

   Vector2f v2f = { 1.1, 2.2 };
   cout << setw(40) << "v2f: " << getstring2v(v2f, s1) << endl;
   normalize2v(v2f);
   cout << setw(40) << "normalize2v(v2f): " << getstring2v(v2f, s1) << endl;
   scale2v(normalize2v(v2f), 2.0);
   cout << setw(40) << "scale(normalize2v(v2f), 2.0): " << getstring2v(v2f, s1) << endl;
   
   Vector3f v3f = { 1.1f, 2.2f, 3.3f };
   cout << setw(40) << "v3f: " << getstring3v(v3f, s1) << endl;
   normalize3v(v3f);
   cout << setw(40) << "normalize3v(v3f): " << getstring3v(v3f, s1) << endl;
   scale3v(normalize3v(v3f), 2.0);
   cout << setw(40) << "scale(normalize3v(v3f), 2.0): " << getstring3v(v3f, s1) << endl;
   
   Vector4d v4d = { 1.1, 2.2, 3.3, 4.4 };
   cout << setw(40) << "v4d: " << getstring4v(v4d, s1) << endl;
   normalize4v(v4d);
   cout << setw(40) << "normalize4v(v4d): " << getstring4v(v4d, s1) << endl;
   scale4v(normalize4v(v4d), 2.0);
   cout << setw(40) << "scale4v(normalize4v(v4d), 2.0): " << getstring4v(v4d, s1) << endl;
   
   }

   // Vector::cross()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::cross()" << endl;

   Vector3f v3f[3] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f }, { 0.0f, 0.0f, 0.0f } };
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   cross(v3f[0], v3f[1], v3f[2]);
   cout << setw(40) << "cross(v3f[0], v3f[1], v3f[2]): " << getstring3v(v3f[2], s1) << endl;

   Vector3d v3d[3] = { { 1.1, 2.2, 3.3 }, { 4.4, 5.5, 6.6 }, { 0.0, 0.0, 0.0 } };
   cout << setw(40) << "v3d[0]: " << getstring3v(v3d[0], s1) << endl;
   cout << setw(40) << "v3d[1]: " << getstring3v(v3d[1], s1) << endl;
   cross(v3d[0], v3d[1], v3d[2]);
   cout << setw(40) << "cross(v3d[0], v3d[1], v3d[2]): " << getstring3v(v3d[2], s1) << endl;

   }


   // Vector::cross()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::get_angle*()" << endl;

   Vector3i v3i[2] = { { 1, 2, 3 }, { 4, 5, 6 } };
   cout << setw(40) << "v3i[0]: " << getstring3v(v3i[0], s1) << endl;
   cout << setw(40) << "v3i[1]: " << getstring3v(v3i[1], s1) << endl;
   cout << setw(40) << "get_angle3v(v3i[0], v3i[1]): " << get_angle3v(v3i[0], v3i[1]) << endl;

   Vector3f v3f[2] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f } };
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   cout << setw(40) << "get_angle3v(v3f[0], v3f[1]): " << get_angle3v(v3f[0], v3f[1]) << endl;
   normalize3v(v3f[0]);
   normalize3v(v3f[1]);
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   cout << setw(40) << "get_angle3n(v3f[0], v3f[1]): " << get_angle3n(v3f[0], v3f[1]) << endl;

   Vector3d v3d[2] = { { 1.1, 2.2, 3.3 }, { 4.4, 5.5, 6.6 } };
   cout << setw(40) << "v3d[0]: " << getstring3v(v3d[0], s1) << endl;
   cout << setw(40) << "v3d[1]: " << getstring3v(v3d[1], s1) << endl;
   cout << setw(40) << "get_angle3v(v3d[0], v3d[1]): " << get_angle3v(v3d[0], v3d[1]) << endl;
   normalize3v(v3d[0]);
   normalize3v(v3d[1]);
   cout << setw(40) << "v3d[0]: " << getstring3v(v3d[0], s1) << endl;
   cout << setw(40) << "v3d[1]: " << getstring3v(v3d[1], s1) << endl;
   cout << setw(40) << "get_angle3n(v3d[0], v3d[1]): " << get_angle3n(v3d[0], v3d[1]) << endl;
   }

   // Vector::subtract*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::subtract*()" << endl;

   Vector2i v2i[3] = { { 1, 2 }, { 3, 4 } };
   cout << setw(40) << "v2i[0]: " << getstring2v(v2i[0], s1) << endl;
   cout << setw(40) << "v2i[1]: " << getstring2v(v2i[1], s1) << endl;
   subtract2v(v2i[0], v2i[1]);
   cout << setw(40) << "subtract2v(v2i[0], v2i[1]): " << getstring2v(v2i[0], s1) << endl;
   cout << setw(40) << "length2v(subtract2v(v2i[0], v2i[1])): " << length2v(subtract2v(v2i[0], v2i[1])) << endl;

   Vector3f v3f[3] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f } };
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   subtract3v(v3f[0], v3f[1]);
   cout << setw(40) << "subtract3v(v3f[0], v3f[1]): " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "length3v(subtract3v(v3f[0], v3f[1])): " << length3v(subtract3v(v3f[0], v3f[1])) << endl;

   Vector4d v4d[3] = { { 1.1, 2.2, 3.3, 4.4 }, { 5.5, 6.6, 7.7, 8.8 } };
   cout << setw(40) << "v4d[0]: " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "v4d[1]: " << getstring4v(v4d[1], s1) << endl;
   subtract4v(v4d[0], v4d[1]);
   cout << setw(40) << "subtract4v(v4d[0], v4d[1]): " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "length4v(subtract4v(v4d[0], v4d[1])): " << length4v(subtract4v(v4d[0], v4d[1])) << endl;

   }

   // Vector::assign*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::assign*()" << endl;

   Vector2i v2i[3] = { { 1, 2 }, { 3, 4 } };
   cout << setw(40) << "v2i[0]: " << getstring2v(v2i[0], s1) << endl;
   cout << setw(40) << "v2i[1]: " << getstring2v(v2i[1], s1) << endl;
   assign2v(v2i[0], v2i[1]);
   cout << setw(40) << "assign2v(v2i[0], v2i[1]): " << getstring2v(v2i[0], s1) << endl;

   Vector3f v3f[3] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f } };
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   assign3v(v3f[0], v3f[1]);
   cout << setw(40) << "assign3v(v3f[0], v3f[1]): " << getstring3v(v3f[0], s1) << endl;

   Vector4d v4d[3] = { { 1.1, 2.2, 3.3, 4.4 }, { 5.5, 6.6, 7.7, 8.8 } };
   cout << setw(40) << "v4d[0]: " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "v4d[1]: " << getstring4v(v4d[1], s1) << endl;
   assign4v(v4d[0], v4d[1]);
   cout << setw(40) << "assign4v(v4d[0], v4d[1]): " << getstring4v(v4d[0], s1) << endl;

   }

   // Vector::proj*()
   {
   cout << "===============================================================" << endl <<
	 "Testing Vector::proj*()" << endl;

   Vector2f v2f[3] = { { 1.1f, 2.2f }, { 3.3f, 4.4f } };
   Vector2f v2fres;
   cout << setw(40) << "v2f[0]: " << getstring2v(v2f[0], s1) << endl;
   cout << setw(40) << "v2f[1]: " << getstring2v(v2f[1], s1) << endl;
   proj3v(v2f[0], v2f[1], v2fres);
   cout << setw(40) << "proj3v(v2f[0], v2f[1], v2fres): " << getstring2v(v2fres, s1) << endl;

   Vector3f v3f[3] = { { 1.1f, 2.2f, 3.3f }, { 4.4f, 5.5f, 6.6f } };
   Vector3f v3fres;
   cout << setw(40) << "v3f[0]: " << getstring3v(v3f[0], s1) << endl;
   cout << setw(40) << "v3f[1]: " << getstring3v(v3f[1], s1) << endl;
   proj3v(v3f[0], v3f[1], v3fres);
   cout << setw(40) << "proj3v(v3f[0], v3f[1], v3fres): " << getstring3v(v3fres, s1) << endl;

   Vector4d v4d[3] = { { 1.1, 2.2, 3.3, 4.4 }, { 5.5, 6.6, 7.7, 8.8 } };
   Vector4d v4dres;
   cout << setw(40) << "v4d[0]: " << getstring4v(v4d[0], s1) << endl;
   cout << setw(40) << "v4d[1]: " << getstring4v(v4d[1], s1) << endl;
   proj4v(v4d[0], v4d[1], v4dres);
   cout << setw(40) << "proj4v(v4d[0], v4d[1], v4dres): " << getstring4v(v4dres, s1) << endl;

   }

   }

////////////////////////////////////////////////////////////////////////////////
void test_matrix(void)
   {
   std::string s;

   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::assign*()" << endl;
   Matrix2i m2i[2] = { { 0, 0, 
			 0, 0 }, 
		       { 1, 2, 
			 3, 4 } };
   Matrix3f m3f[2] = { { 0.0f, 0.0f, 0.0f, 
			 0.0f, 0.0f, 0.0f, 
			 0.0f, 0.0f, 0.0f }, 
		       { 1.1f, 2.2f, 3.3f, 
			 4.4f, 5.5f, 6.6f, 
			 7.7f, 8.8f, 9.9f } };
   Matrix4d m4d[2] = { { 0.0, 0.0, 0.0, 0.0, 
			 0.0, 0.0, 0.0, 0.0, 
			 0.0, 0.0, 0.0, 0.0, 
			 0.0, 0.0, 0.0, 0.0 }, 
		       { 1.1, 2.2, 3.3, 4.4, 
			 5.5, 6.6, 7.7, 8.8, 
			 9.9, 10.10, 11.11, 12.12,
			 13.13, 14.14, 15.15, 16.16 } };
   cout << setw(40) << "m2i[0]" << getstring2m(m2i[0], s) << endl;
   cout << setw(40) << "m2i[1]" << getstring2m(m2i[1], s) << endl;
   assign2m(m2i[0], m2i[1]);
   cout << setw(40) << "assign2m(m2i[0], m2i[1])" << getstring2m(m2i[0], s) << endl;

   cout << setw(40) << "m3f[0]" << getstring3m(m3f[0], s) << endl;
   cout << setw(40) << "m3f[1]" << getstring3m(m3f[1], s) << endl;
   assign3m(m3f[0], m3f[1]);
   cout << setw(40) << "assign3m(m3f[0], m3f[1])" << getstring3m(m3f[0], s) << endl;

   cout << setw(40) << "m4d[0]" << getstring4m(m4d[0], s) << endl;
   cout << setw(40) << "m4d[1]" << getstring4m(m4d[1], s) << endl;
   assign4m(m4d[0], m4d[1]);
   cout << setw(40) << "assign4m(m3d[0], m3d[1])" << getstring4m(m4d[0], s) << endl;
   }

   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::add*()" << endl;
   Matrix2i m2i[2] = { { 1, 2, 
			 3, 4 },
		       { 5, 6, 
			 7, 8 } };
   Matrix3f m3f[2] = { { 1.1f, 2.2f, 3.3f, 
			 4.4f, 5.5f, 6.6f, 
			 7.7f, 8.8f, 9.9f },
		       { 10.10f, 11.11f, 12.12f, 
			 13.13f, 14.14f, 15.15f, 
			 16.16f, 17.17f, 18.18f } };
   Matrix4d m4d[2] = { { 1.1, 2.2, 3.3, 4.4, 
			 5.5, 6.6, 7.7, 8.8, 
			 9.9, 10.10, 11.11, 12.12,
			 13.13, 14.14, 15.15, 16.16 },
		       { 17.17, 18.18, 19.19, 20.20, 
			 21.21, 22.22, 23.23, 24.24, 
			 25.25, 26.26, 27.27, 28.28,
			 29.29, 30.30, 31.31, 32.32 } };
   cout << setw(40) << "m2i[0]" << getstring2m(m2i[0], s) << endl;
   cout << setw(40) << "m2i[1]" << getstring2m(m2i[1], s) << endl;
   add2m(m2i[0], m2i[1]);
   cout << setw(40) << "add2m(m2i[0], m2i[1])" << getstring2m(m2i[0], s) << endl;

   cout << setw(40) << "m3f[0]" << getstring3m(m3f[0], s) << endl;
   cout << setw(40) << "m3f[1]" << getstring3m(m3f[1], s) << endl;
   add3m(m3f[0], m3f[1]);
   cout << setw(40) << "add3m(m3f[0], m3f[1])" << getstring3m(m3f[0], s) << endl;

   cout << setw(40) << "m4d[0]" << getstring4m(m4d[0], s) << endl;
   cout << setw(40) << "m4d[1]" << getstring4m(m4d[1], s) << endl;
   add4m(m4d[0], m4d[1]);
   cout << setw(40) << "add4m(m3d[0], m3d[1])" << getstring4m(m4d[0], s) << endl;
   }

   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::subtract*()" << endl;
   Matrix2i m2i[2] = { { 1, 2, 
			 3, 4 },
		       { 5, 6, 
			 7, 8 } };
   Matrix3f m3f[2] = { { 1.1f, 2.2f, 3.3f, 
			 4.4f, 5.5f, 6.6f, 
			 7.7f, 8.8f, 9.9f },
		       { 10.10f, 11.11f, 12.12f, 
			 13.13f, 14.14f, 15.15f, 
			 16.16f, 17.17f, 18.18f } };
   Matrix4d m4d[2] = { { 1.1, 2.2, 3.3, 4.4, 
			 5.5, 6.6, 7.7, 8.8, 
			 9.9, 10.10, 11.11, 12.12,
			 13.13, 14.14, 15.15, 16.16 },
		       { 17.17, 18.18, 19.19, 20.20, 
			 21.21, 22.22, 23.23, 24.24, 
			 25.25, 26.26, 27.27, 28.28,
			 29.29, 30.30, 31.31, 32.32 } };
   cout << setw(40) << "m2i[0]" << getstring2m(m2i[0], s) << endl;
   cout << setw(40) << "m2i[1]" << getstring2m(m2i[1], s) << endl;
   subtract2m(m2i[0], m2i[1]);
   cout << setw(40) << "subtract2m(m2i[0], m2i[1])" << getstring2m(m2i[0], s) << endl;

   cout << setw(40) << "m3f[0]" << getstring3m(m3f[0], s) << endl;
   cout << setw(40) << "m3f[1]" << getstring3m(m3f[1], s) << endl;
   subtract3m(m3f[0], m3f[1]);
   cout << setw(40) << "subtract3m(m3f[0], m3f[1])" << getstring3m(m3f[0], s) << endl;

   cout << setw(40) << "m4d[0]" << getstring4m(m4d[0], s) << endl;
   cout << setw(40) << "m4d[1]" << getstring4m(m4d[1], s) << endl;
   subtract4m(m4d[0], m4d[1]);
   cout << setw(40) << "subtract4m(m3d[0], m3d[1])" << getstring4m(m4d[0], s) << endl;
   }

   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::scale*()" << endl;
   Matrix2i m2i = { 1, 2, 
		    3, 4 };
   Matrix3f m3f = { 1.1f, 2.2f, 3.3f, 
		    4.4f, 5.5f, 6.6f, 
		    7.7f, 8.8f, 9.9f };
   Matrix4d m4d = { 1.1, 2.2, 3.3, 4.4, 
		    5.5, 6.6, 7.7, 8.8, 
		    9.9, 10.10, 11.11, 12.12,
		    13.13, 14.14, 15.15, 16.16 };
   cout << setw(40) << "m2i" << getstring2m(m2i, s) << endl;
   cout << setw(40) << "scale2m(m2i, 2)" << getstring2m(scale2m(m2i, 2), s) << endl;

   cout << setw(40) << "m3f" << getstring3m(m3f, s) << endl;
   cout << setw(40) << "scale3m(m3f, 2.2f)" << getstring3m(scale3m(m3f, 2.2f), s) << endl;

   cout << setw(40) << "m4d" << getstring4m(m4d, s) << endl;
   cout << setw(40) << "scale4m(m3d, 2.2)" << getstring4m(scale4m(m4d, 2.2), s) << endl;
   }

   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::multvec*()" << endl;
   Vector2i v2ires;
   Vector3f v3fres;
   Vector4d v4dres;
   Vector2i v2i = { 1, 2 };
   Vector3f v3f = { 1.1f, 2.2f, 3.3f };
   Vector4d v4d = { 1.1, 2.2, 3.3, 4.4 };
   Matrix2i m2i = { 1, 2, 
		    3, 4 };
   Matrix3f m3f = { 1.1f, 2.2f, 3.3f, 
		    4.4f, 5.5f, 6.6f, 
		    7.7f, 8.8f, 9.9f };
   Matrix4d m4d = { 1.1, 2.2, 3.3, 4.4, 
		    5.5, 6.6, 7.7, 8.8, 
		    9.9, 10.10, 11.11, 12.12,
		    13.13, 14.14, 15.15, 16.16 };
   cout << setw(40) << "v2i" << getstring2v(v2i, s) << endl;
   cout << setw(40) << "m2i" << getstring2m(m2i, s) << endl;
   cout << setw(40) << "multvec2mv(m2i, v2i, v2ires)" << 
	 getstring2v(multvec2mv(m2i, v2i, v2ires), s) << endl;
   cout << setw(40) << "multvec2mv(m2i, v2i, v2ires)" << 
	 getstring2v(multvec2vm(v2i, m2i, v2ires), s) << endl;

   cout << setw(40) << "v3f" << getstring3v(v3f, s) << endl;
   cout << setw(40) << "m3f" << getstring3m(m3f, s) << endl;
   cout << setw(40) << "multvec3mv(m3f, v3f, v3fres)" << 
	 getstring3v(multvec3mv(m3f, v3f, v3fres), s) << endl;
   cout << setw(40) << "multvec3mv(m3f, v3f, v3fres)" << 
	 getstring3v(multvec3vm(v3f, m3f, v3fres), s) << endl;

   cout << setw(40) << "v4d" << getstring4v(v4d, s) << endl;
   cout << setw(40) << "m4d" << getstring4m(m4d, s) << endl;
   cout << setw(40) << "multvec4mv(m4d, v4d, v4dres)" << 
	 getstring4v(multvec4mv(m4d, v4d, v4dres), s) << endl;
   cout << setw(40) << "multvec4mv(m4d, v4d, v4dres)" << 
	 getstring4v(multvec4vm(v4d, m4d, v4dres), s) << endl;

   }


   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::multmat*()" << endl;
   Matrix2i m2ires;
   Matrix3f m3fres;
   Matrix4d m4dres;
   Matrix2i m2i[2] = { { 1, 2, 
			 3, 4 },
		       { 5, 6, 
			 7, 8 } };
   Matrix3f m3f[2] = { { 1.1f, 2.2f, 3.3f, 
			 4.4f, 5.5f, 6.6f, 
			 7.7f, 8.8f, 9.9f },
		       { 10.10f, 11.11f, 12.12f, 
			 13.13f, 14.14f, 15.15f, 
			 16.16f, 17.17f, 18.18f } };
   Matrix4d m4d[2] = { { 1.1, 2.2, 3.3, 4.4, 
			 5.5, 6.6, 7.7, 8.8, 
			 9.9, 10.10, 11.11, 12.12,
			 13.13, 14.14, 15.15, 16.16 },
		       { 17.17, 18.18, 19.19, 20.20, 
			 21.21, 22.22, 23.23, 24.24, 
			 25.25, 26.26, 27.27, 28.28,
			 29.29, 30.30, 31.31, 32.32 } };
   cout << setw(40) << "m2i[0]" << getstring2m(m2i[0], s) << endl;
   cout << setw(40) << "m2i[1]" << getstring2m(m2i[1], s) << endl;
   multmat2(m2i[0], m2i[1], m2ires);
   cout << setw(40) << "multmat2(m2i[0], m2i[1], m21res)" << 
	 getstring2m(m2ires, s) << endl;

   cout << setw(40) << "m3f[0]" << getstring3m(m3f[0], s) << endl;
   cout << setw(40) << "m3f[1]" << getstring3m(m3f[1], s) << endl;
   multmat3(m3f[0], m3f[1], m3fres);
   cout << setw(40) << "multmat3(m3f[0], m3f[1], m3fres)" << 
	 getstring3m(m3fres, s) << endl;

   cout << setw(40) << "m4d[0]" << getstring4m(m4d[0], s) << endl;
   cout << setw(40) << "m4d[1]" << getstring4m(m4d[1], s) << endl;
   multmat4(m4d[0], m4d[1], m4dres);
   cout << setw(40) << "multmat4(m4d[0], m4d[1], m4dres)" << 
	 getstring4m(m4dres, s) << endl;
   }

   {
   cout << "===============================================================" << endl <<
	 "Testing Matrix::transpose*()" << endl;
   Matrix2i m2ires;
   Matrix3f m3fres;
   Matrix4d m4dres;
   Matrix2i m2i = { 1, 2, 
		    3, 4 };
   Matrix3f m3f = { 1.1f, 2.2f, 3.3f, 
		    4.4f, 5.5f, 6.6f, 
		    7.7f, 8.8f, 9.9f };
   Matrix4d m4d = { 1.1, 2.2, 3.3, 4.4, 
		    5.5, 6.6, 7.7, 8.8, 
		    9.9, 10.10, 11.11, 12.12,
		    13.13, 14.14, 15.15, 16.16 };
   cout << setw(40) << "m2i" << getstring2m(m2i, s) << endl;
   cout << setw(40) << "transpose2(m2i, m2ires)" << 
	 getstring2m(transpose2(m2i, m2ires), s) << endl;

   cout << setw(40) << "m3f" << getstring3m(m3f, s) << endl;
   cout << setw(40) << "transpose3(m3f, m3fres)" << 
	 getstring3m(transpose3(m3f, m3fres), s) << endl;

   cout << setw(40) << "m4d" << getstring4m(m4d, s) << endl;
   cout << setw(40) << "transpose4(m4d, m4dres)" << 
	 getstring4m(transpose4(m4d, m4dres), s) << endl;

   }
   }

////////////////////////////////////////////////////////////////////////////////
void test_all(void)
   {
   test_vector();
   test_matrix();
   }

////////////////////////////////////////////////////////////////////////////////
void show_menu(void)
   {
   cout << endl << endl
	<< "Test what?" << endl
	<< "0)      All" << endl << endl
	<< "1)      Vector" << endl
	<< "2)      Matrix" << endl
	<< endl
	<< "99      Quit" << endl;
   }

////////////////////////////////////////////////////////////////////////////////
int main (int argc, char * argv[]) 
   {
   int retval = 0;

   try
      {
      int choice = -1;
      
      while (choice != 99)
	 {
	 show_menu();
	 cin >> choice;
	 if(!cin)
	    {
	    cin.clear();
	    cin.ignore(std::numeric_limits<std::streamsize>::max(),'\n');
	    }
	 if (choice != 99)
	    {
	    switch (choice)
	       {
	       case 0:
		  test_all();
		  break;
	       case 1:
		  test_vector();
		  break;
	       case 2:
		  test_matrix();
		  break;
	       default:
		  cout << "Unrecognized choice.  Please try again." << endl;
	       }
	    choice = -1;
	    }
	 }
      }
   catch (const std::exception & error)
      {
      string e = "Caught exception: ";
      e += error.what();
      cerr << e << endl;
      retval = 1;
      }

   return retval;;

   }
